#   Copyright (c) 2014 - 2017 Axel Wachtler
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the authors nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.


# === imports ================================================================
from __future__ import print_function
from dbmysql import DbMySQL
from dbsqlite import DbSqlite
import rsensor
import os

# === globals ================================================================
DB_SQLITE = "sqlite"
DB_MYSQL  = "mysql"

def db_open(**cfg):
    dbtype = cfg.get("dbtype", DB_SQLITE)
    data_dir = os.path.abspath(os.path.dirname(rsensor.__file__)) + "/data"
    assert os.path.exists(data_dir)

    rv = None
    if dbtype == DB_SQLITE:
        rv = DbSqlite(dbschema = cfg.get("dbschema", data_dir + "/schema.yml"),
                      dbname = cfg['dbname'])
    elif dbtype == DB_MYSQL:
        rv = DbMySQL(dbschema = cfg.get("dbschema", data_dir + "/schema.yml"),
                     dbname = cfg["dbname"],
                     dbhost = cfg.get("dbhost", "localhost"),
                     dbuser = cfg.get("dbuser", "rsensor"),
                     dbpasswd = cfg.get("dbpasswd", None))
    return rv


